from typing import Any, Union, List


class CaseBuilder:

    _query: List[dict]
    _context: List[dict]
    _response: List[dict]
    _metadata: dict

    def __init__(self, client):
        self._client = client
        self._query = []
        self._context = []
        self._response = []
        self._metadata = {}

    def add_query(self, name: str, value: Any, type: Union[str, dict]):
        self._query.append({
            "name": name,
            "value": value,
            "type": type
        })
        return self

    def add_context(self, name: str, value: Any, type: Union[str, dict]):
        self._context.append({
            "name": name,
            "value": value,
            "type": type
        })
        return self

    def add_response(self, name: str, value: Any, type: Union[str, dict], evaluation: List[dict] = None):
        self._response.append({
            "name": name,
            "value": value,
            "type": type,
            "evaluation": evaluation or []
        })
        return self

    def add_evaluation(self, question: str, name: str, **kwargs):
        self._response[-1]["evaluation"].append({
            "question": question,
            "name": name,
            **kwargs
        })
        return self

    def set_metadata(self, **kwargs):
        self._metadata = kwargs
        return self

    def send(self):
        self._client.send({
            "query": self._query,
            "context": self._context,
            "response": self._response,
            "metadata": self._metadata
        })




import json
import logging
from datetime import datetime, timedelta
from time import sleep
from urllib.parse import urlencode

import requests

from catwalk_common import OpenCase

logger = logging.getLogger("catwalk_client")


class CaseExporter:

    def __init__(self, from_datetime: datetime, to_datetime: datetime, catwalk_url: str,
                 submitter_name: str = None, submitter_version: str = None, max_retries: int = 5, batch: int = 1):
        self.from_datetime = from_datetime
        self.to_datetime = to_datetime
        self.catwalk_url = catwalk_url

        self.filters = self._build_filters(submitter_name, submitter_version, from_datetime, to_datetime)
        self.max_retries = max_retries
        self.batch = batch

    def export(self):
        for date in self._hour_range():
            current_retry = 0
            path = f"/api/cases/export/{date.year}/{date.month}/{date.day}/{date.hour}"

            if self.filters:
                path += f'?{urlencode(self.filters)}'

            next_token = self._get_url(path)

            while next_token:
                response = requests.get(next_token)

                if not response.ok:
                    self._handle_retry(current_retry, response)
                    current_retry += 1
                    continue

                next_token = None
                data = json.loads(response.text)
                for item in data['items']:
                    yield OpenCase.parse_obj(item)

                if data['next_part'] and data['items']:
                    next_token = data['next_part']

    def _handle_retry(self, current_retry: int, response):
        if current_retry < self.max_retries:
            multiplier = (current_retry * 2) or 1
            logger.info(f"[catwalk-export] Retry ({current_retry + 1}/{self.max_retries}) for url {response.url} "
                        f"because got error {response.status_code}")
            sleep(0.5 * multiplier)
        else:
            raise Exception("Max retries exceeded")

    @staticmethod
    def _build_filters(submitter_name: str, submitter_version: str, from_datetime: datetime, to_datetime: datetime):
        filters = {
            "submitter_name": submitter_name,
            "submitter_version": submitter_version,
            "from_timestamp": from_datetime.isoformat(),
            "to_timestamp": to_datetime.isoformat()
        }
        return {k: v for k, v in filters.items() if v is not None}

    def _get_url(self, path: str):
        return self.catwalk_url.rstrip('/') + path

    def _hour_range(self):
        start, end = self.from_datetime, self.to_datetime

        while start < end:
            yield start
            start += timedelta(hours=1)



import json
import logging
import os
from datetime import datetime, timedelta

import requests
from ._case_builder import CaseBuilder
from ._case_exporter import CaseExporter
from catwalk_common import CommonCaseFormat

logger = logging.getLogger("catwalk_client")


def _hour_range(start, end):
    while start < end:
        yield start
        start += timedelta(hours=1)


class CatwalkClient:
    submitter_name: str
    submitter_version: str
    catwalk_url: str

    def __init__(self, submitter_name: str = None, submitter_version: str = None, catwalk_url: str = None):
        self.submitter_name = submitter_name
        self.submitter_version = submitter_version
        self.catwalk_url = catwalk_url or os.environ.get("CATWALK_URL")

    def new_case(self) -> CaseBuilder:
        return CaseBuilder(client=self)

    def _get_url(self, path: str):
        return self.catwalk_url.rstrip('/') + path

    def send(self, case: dict):
        case = CommonCaseFormat(
            submitter={"name": self.submitter_name, "version": self.submitter_version},
            **case
        )

        response = requests.post(self._get_url("/api/cases/collect"), data=case.json())

        if response.ok:
            data = json.loads(response.text)
            logger.info(f"Collected catwalk case: {data['id']}")

    def export_cases(self, from_datetime: datetime, to_datetime: datetime,
                     submitter_name: str = None, submitter_version: str = None, max_retries: int = 5):
        exporter = CaseExporter(from_datetime=from_datetime, to_datetime=to_datetime, catwalk_url=self.catwalk_url,
                                submitter_name=submitter_name, submitter_version=submitter_version,
                                max_retries=max_retries)
        yield from exporter.export()



import setuptools

with open("requirements.txt", "r") as fh:
    install_requires = fh.read().splitlines()

setuptools.setup(
    name="catwalk_client",
    version='0.0.5',
    author='ERST',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8.0',
    install_requires=install_requires,
    dependency_links=[],
)
